@@ -12,7 +12,7 @@
 
    ---------------------------------------------------------------------------
 
-   Copyright (C) 2003,2004  Volker C. Behr
+   Copyright (C) 2003,2004,2005  Volker C. Behr
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -38,45 +38,54 @@
 
   
    HISTORY: 
-   10/07/2004 : 1.6.3      - corrected erroneous call of GhostScript
-   10/05/2004 : 1.6.2      - added necessary quotes in the call of GhostScript
-   10/03/2004 : 1.6.1      - allow users w/o valid shell in /etc/passwd,
+   2005-03-07 : 1.7.0a
+   2005-03-01 : 1.7.0      - call of GhostScript from a non-privileged child process,
+                             no more additional programs in the call of GhostScript,
                              several code optimizations
-   09/08/2004 : 1.6.0      - new option to check against user names in lower case,
+   2005-02-09 : 1.6.6      - updated PostScript PPD file supporting more DIN formats
+   2005-01-04 : 1.6.5      - cups-pdf is running with gid of group defined in CPGRP
+   2004-10-11 : 1.6.4      - improved configurability for different versions of 'su'
+   2004-10-07 : 1.6.3      - corrected erroneous call of GhostScript
+   2004-10-05 : 1.6.2      - added necessary quotes in the call of GhostScript
+   2004-10-03 : 1.6.1      - allow users w/o valid shell in /etc/passwd,
+                             several code optimizations
+   2004-09-08 : 1.6.0      - new option to check against user names in lower case,
                              new option to label all created PDFs with a job-id
-   08/10/2004 : 1.5.2      - fixed insecure creation of spoolfile,
+   2004-08-10 : 1.5.2      - fixed insecure creation of spoolfile,
      			     several smaller code optimizations
-   08/09/2004 : 1.5.1	   - fixed possible access to element outside allocated area,
+   2004-08-09 : 1.5.1	   - fixed possible access to element outside allocated area,
                              fixed possible conflict with overlapping memory areas
-   08/03/2004 : 1.5.0      - call to GS without root privileges for enhanced security
-   08/01/2004 : 1.4.3      - added -dSAFER to the call to GS to improve security
-   02/24/2004 : 1.4.2      - improved extraction of filenames,
+   2004-08-03 : 1.5.0      - call to GS without root privileges for enhanced security
+   2004-08-01 : 1.4.3      - added -dSAFER to the call to GS to improve security
+   2004-02-25 : 1.4.2a
+   2004-02-24 : 1.4.2      - improved extraction of filenames,
                              handling of embedded (e)ps code (RVT)
-   02/09/2004 : 1.4.1	   - logfile is created with mode 644 if it does not exist
-   01/27/2004 : 1.4.0      - new option to set permissions for PDF output,
+   2004-02-09 : 1.4.1	   - logfile is created with mode 644 if it does not exist
+   2004-01-27 : 1.4.0      - new option to set permissions for PDF output,
                              alternative options for improved image conversion
                              (moved CPGSCALL to cups-pdf.h)
-   01/06/2004 : 1.3.2	   - exit status of ghostscript gets logged in debug mode
-   11/29/2003 : 1.3.1	   - improved handling of domain prefix
-   11/02/2003 : 1.3	   - new option to cut file name extensions
-   10/25/2003 : 1.2        - untitled documents are labelled with job-id,
+   2004-01-06 : 1.3.2	   - exit status of ghostscript gets logged in debug mode
+   2004-01-02 : 1.3.1a
+   2003-11-29 : 1.3.1	   - improved handling of domain prefix
+   2003-11-02 : 1.3	   - new option to cut file name extensions
+   2003-10-25 : 1.2        - untitled documents are labelled with job-id,
 			     new option to add a domain prefix to user names,
                              removed a minor bug during character replacement
-   08/02/2003 : 1.1	   - files not viewable until creation is finished,
+   2003-08-02 : 1.1	   - files not viewable until creation is finished,
    			     new qualifier "$HOME" for output directory,
 			     "$HOME"-related new define in the header file
-   07/17/2003 : 1.0.1	   - improved user name handling for multi-word names,
+   2003-07-17 : 1.0.1	   - improved user name handling for multi-word names,
                              code now distributed under the GPL
-   06/29/2003 : 1.0	   - enhanced debug output,
+   2003-06-29 : 1.0	   - enhanced debug output,
    			     replacement of special characters in title,
 			     new option in the header file
-   05/26/2003 : 1.0pre3    - several minor bugfixes and improvements 
-   05/21/2003 : 1.0pre2    - optimized memory management
-   05/16/2003 : 1.0pre1    - introduced cups-pdf.h, 
+   2003-05-26 : 1.0pre3    - several minor bugfixes and improvements 
+   2003-05-21 : 1.0pre2    - optimized memory management
+   2003-05-16 : 1.0pre1    - introduced cups-pdf.h, 
 			     code now entirely written in C,
  			     cups-pdfgen is obsolete
-   02/14/2003 : 0.2    - script re-located to /usr/lib/cups/filter/cups-pdfgen
-   02/12/2003 : 0.1    - first beta release			                */
+   2003-02-14 : 0.2    - script re-located to /usr/lib/cups/filter/cups-pdfgen
+   2003-02-12 : 0.1    - first beta release			                */
 
 #define CPERROR 	1
 #define CPSTATUS 	2
@@ -95,15 +104,18 @@
 #include <unistd.h>
 #include <stddef.h>
 #include <pwd.h>
+#include <grp.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 
 #include "cups-pdf.h"
 
 extern int errno;
 
+FILE *logfp;
+
 void log_event(char type, char message[48], char *detail) {
-  FILE *fp;
   time_t secs;
   char ctype[8], *timestring;
 
@@ -118,33 +130,45 @@ void log_event(char type, char message[48], char *detail) {
       sprintf(ctype, "STATUS");
     else
       sprintf(ctype, "DEBUG");
-    fp=fopen(CPLOG"/cups-pdf_log", "a");
     if (detail != NULL)  {
       while (detail[strlen(detail)-1] == '\n') 
         detail[strlen(detail)-1]='\0';
-      fprintf(fp,"%s  [%s] %s (%s)\n", timestring, ctype, message, detail);
+      fprintf(logfp,"%s  [%s] %s (%s)\n", timestring, ctype, message, detail);
       if ((CPLOGTYPE & CPDEBUG) && (type & CPERROR)) 
-        fprintf(fp,"%s  [DEBUG] ERRNO: %d\n", timestring, errno);
+        fprintf(logfp,"%s  [DEBUG] ERRNO: %d\n", timestring, errno);
     }
     else
-      fprintf(fp,"%s  [%s] %s\n", timestring, ctype, message);
-    fclose(fp);
+      fprintf(logfp,"%s  [%s] %s\n", timestring, ctype, message);
   }
   return;
 }
     
 int init() {
   struct stat fstatus;
+  struct group *group;
 
   umask(0077);
+
+  group=getgrnam(CPGRP);
+  if (group)
+    setgid(group->gr_gid);
+
   if (strlen(CPLOG)) {
     if (stat(CPLOG, &fstatus) || !S_ISDIR(fstatus.st_mode)) {
       if (mkdir(CPLOG, 0700))
         return 1;
       log_event(CPSTATUS, "log directory created", CPLOG);
     }
-    fopen(CPLOG"/cups-pdf_log", "a");
+    logfp=fopen(CPLOG"/cups-pdf_log", "a");
   }
+
+  if (!group) {
+    log_event(CPERROR, "CPGRP not found", CPGRP);
+    return 1;
+  }
+  else 
+    log_event(CPDEBUG, "switching to new gid", CPGRP);
+
   umask(0022);
   if (!strncmp(CPOUT, "$HOME", 5)) 
     log_event(CPDEBUG, "output redirected to home directory", "$HOME");
@@ -155,6 +179,7 @@ int init() {
     }
     log_event(CPSTATUS, "output directory created", CPOUT);
   }
+
   if (stat(CPSPOOL, &fstatus) || !S_ISDIR(fstatus.st_mode)) {
     if (mkdir(CPSPOOL, 0755)) {
       log_event(CPERROR, "failed to create spool directory", CPSPOOL);
@@ -162,6 +187,7 @@ int init() {
     }
     log_event(CPSTATUS, "spool directory created", CPSPOOL);
   }
+
   umask(0077);
   return 0;
 }
@@ -225,7 +251,7 @@ int preparespoolfile(FILE *fpsrc, char *spoolfile, char *title, int job, struct
     return 1;
   }
   log_event(CPDEBUG, "destination stream ready", spoolfile);
-  if (chown(spoolfile, passwd->pw_uid, passwd->pw_gid)) {
+  if (chown(spoolfile, passwd->pw_uid, -1)) {
     log_event(CPERROR, "failed to set owner for spoolfile", spoolfile);
     return 1;
   }
@@ -294,8 +320,7 @@ int main(int argc, char *argv[]) {
   int size;
   mode_t mode;
   struct passwd *passwd;
-
-  umask(0077);
+  pid_t pid;
 
   if (init()) return -1;
   log_event(CPDEBUG, "initialization finished", NULL);
@@ -333,6 +358,7 @@ int main(int argc, char *argv[]) {
       if (passwd == NULL) {
         log_event(CPERROR, "username for anonymous access unknown", CPANONUSER);
         free(user);
+        fclose(logfp);
         return -2;
       }
       log_event(CPDEBUG, "unknown user", user);
@@ -342,6 +368,7 @@ int main(int argc, char *argv[]) {
         if (dirname == NULL) {
           fputs("CUPS-PDF: failed to allocate memory\n", stderr);
           free(user);
+          fclose(logfp);
           return -2;
         }  
         snprintf(dirname, size, "%s/%s", passwd->pw_dir, CPHOMESUB);
@@ -352,6 +379,7 @@ int main(int argc, char *argv[]) {
         if (dirname == NULL) {
           fputs("CUPS-PDF: failed to allocate memory\n", stderr);
           free(user);
+          fclose(logfp);
           return -2;
         }  
         snprintf(dirname, size, "%s/%s", CPOUT, CPANONDIRNAME);
@@ -360,6 +388,7 @@ int main(int argc, char *argv[]) {
     else {
       log_event(CPSTATUS, "anonymous access denied", user);
       free(user);
+      fclose(logfp);
       return 0;
     }
     mode=(0666&~CPANONUMASK);
@@ -372,6 +401,7 @@ int main(int argc, char *argv[]) {
       if (dirname == NULL) {
         fputs("CUPS-PDF: failed to allocate memory\n", stderr);
         free(user);
+        fclose(logfp);
         return -2;
       }  
       snprintf(dirname, size, "%s/%s", passwd->pw_dir, CPHOMESUB);
@@ -382,6 +412,7 @@ int main(int argc, char *argv[]) {
       if (dirname == NULL) {
         fputs("CUPS-PDF: failed to allocate memory\n", stderr);
         free(user);
+        fclose(logfp);
         return -2;
       }  
       if (!CPDIRPREFIX)
@@ -400,6 +431,7 @@ int main(int argc, char *argv[]) {
   if (spoolfile == NULL) {
     fputs("CUPS-PDF: failed to allocate memory\n", stderr);
     free(dirname);
+    fclose(logfp);
     return -3;
   }
   snprintf(spoolfile, size, "%s/cups2pdf-%i", CPSPOOL, (int) getpid());
@@ -409,6 +441,7 @@ int main(int argc, char *argv[]) {
     if (preparespoolfile(stdin, spoolfile, title, atoi(argv[1]), passwd)) {
       free(dirname);
       free(spoolfile);
+      fclose(logfp);
       return -4;
     }
     log_event(CPDEBUG, "input data read from stdin", NULL);
@@ -417,6 +450,7 @@ int main(int argc, char *argv[]) {
     if (preparespoolfile(fopen(argv[6], "r"), spoolfile, title, atoi(argv[1]), passwd)) {
       free(dirname);
       free(spoolfile);
+      fclose(logfp);
       return -4;
     }
     log_event(CPDEBUG, "input data read from file", argv[6]);
@@ -428,32 +462,66 @@ int main(int argc, char *argv[]) {
     fputs("CUPS-PDF: failed to allocate memory\n", stderr);
     free(dirname);
     free(spoolfile);
+    fclose(logfp);
     return -5;
   }
   snprintf(outfile, size, "%s/%s.pdf", dirname, title);
   log_event(CPDEBUG, "output filename created", outfile);
 
-  size=strlen(passwd->pw_name)+strlen(CPSU)+strlen(CPSH)+strlen(CPGSCALL)+strlen(CPGHOSTSCRIPT)+strlen(CPPDFVER)+strlen(outfile)+strlen(spoolfile)+6;
+  size=strlen(CPGSCALL)+strlen(CPGHOSTSCRIPT)+strlen(CPPDFVER)+strlen(outfile)+strlen(spoolfile)+6;
   gscall=calloc(size, sizeof(char));
   if (gscall == NULL) {
     fputs("CUPS-PDF: failed to allocate memory\n", stderr);
     free(dirname);
     free(spoolfile);
     free(outfile);
+    fclose(logfp);
     return -6;
   }
-  snprintf(gscall, size, CPGSCALL, CPSU, passwd->pw_name, CPSH, CPGHOSTSCRIPT, CPPDFVER, outfile, spoolfile);
+  snprintf(gscall, size, CPGSCALL, CPGHOSTSCRIPT, CPPDFVER, outfile, spoolfile);
   log_event(CPDEBUG, "ghostscript commandline built", gscall);
 
   unlink(outfile);
   log_event(CPDEBUG, "output file unlinked", outfile);
 
-  size=system(gscall);
-  snprintf(title,BUFSIZE,"%d",size);
-  log_event(CPDEBUG, "ghostscript has finished", title);
-  
-  unlink(spoolfile);
-  log_event(CPDEBUG, "spoolfile unlinked", spoolfile);
+  if (setenv("TMPDIR", CPGSTMP, 1)) {
+    log_event(CPERROR, "insufficient space in environment to set TMPDIR", CPGSTMP);
+    free(dirname);
+    free(spoolfile);
+    free(outfile);
+    free(gscall);
+    fclose(logfp);
+    return -7;
+  }
+  log_event(CPDEBUG, "TMPDIR set for GhostScript", getenv("TMPDIR"));
+
+  fflush(logfp);
+  pid=fork();
+
+  if (!pid) {
+    log_event(CPDEBUG, "entering child process", NULL);
+
+    if (setgid(passwd->pw_gid))
+      log_event(CPERROR, "failed to set GID for current user", NULL);
+    else 
+      log_event(CPDEBUG, "GID set for current user", NULL);
+    if (setuid(passwd->pw_uid))
+      log_event(CPERROR, "failed to set UID for current user", passwd->pw_name);
+    else 
+      log_event(CPDEBUG, "UID set for current user", passwd->pw_name);
+
+    size=system(gscall);
+    snprintf(title,BUFSIZE,"%d",size);
+    log_event(CPDEBUG, "ghostscript has finished", title);
+    return 0;
+  }
+  log_event(CPDEBUG, "waiting for child to exit", NULL);
+  waitpid(pid,NULL,0);
+
+  if (unlink(spoolfile)) 
+    log_event(CPERROR, "failed to unlink spoolfile (non fatal)", spoolfile);
+  else 
+    log_event(CPDEBUG, "spoolfile unlinked", spoolfile);
 
   if (chmod(outfile, mode)) {
     log_event(CPERROR, "failed to set file mode for PDF file", outfile);
@@ -461,10 +529,11 @@ int main(int argc, char *argv[]) {
     free(spoolfile);
     free(outfile);
     free(gscall);
-    return -7;
+    fclose(logfp);
+    return -9;
   }
   log_event(CPDEBUG, "file mode set for user output", outfile);
-
+ 
   free(dirname);
   free(spoolfile);
   free(outfile);
@@ -474,5 +543,6 @@ int main(int argc, char *argv[]) {
 
   log_event(CPSTATUS, "PDF creation successfully finished", passwd->pw_name);
 
+  fclose(logfp);
   return 0;
 } 
