@@ -1,4 +1,4 @@
-/* cups-pdf.h -- CUPS Backend Header File (version  1.6.3, 10/07/2004)
+/* cups-pdf.h -- CUPS Backend Header File (version  1.7.0a, 2005-03-07)
    16.05.2003, Volker C. Behr
    Exp. Physik V, Uni Wuerzburg 
    vrbehr@cip.physik.uni-wuerzburg.de
@@ -21,25 +21,15 @@
 /* location of GhostScript binary (gs) */
 #define CPGHOSTSCRIPT "/usr/bin/gs"
 
-/* location of 'su' binary */
-#define CPSU "/bin/su"
+/* location of temporary files during GhostScript operation 
+   this must be user-writable like /var/tmp or /tmp !   	*/
+#define CPGSTMP "/var/tmp"
 
-/* location of the default shell 
-   this must be a valid shell (see /etc/shells) */
-#define CPSH "/bin/sh"
-
-/* command line for calling GhostScript
-   new version for enhanced image quality
-   to use old version un-comment the top command line and comment
-   out the bottom one							*/
-/*   
-#define CPGSCALL "%s - \"%s\" -s %s -c \"%s -q -dCompatibilityLevel=%s -dNOPAUSE -dBATCH \
-        -dSAFER -sDEVICE=pdfwrite -sOutputFile=\"%s\" -c save pop -f %s\""
-*/
-#define CPGSCALL "%s - \"%s\" -s %s -c \"%s -q -dCompatibilityLevel=%s -dNOPAUSE -dBATCH \
-        -dSAFER -sDEVICE=pdfwrite -sOutputFile=\"%s\" -dAutoRotatePages=/PageByPage \
-        -dAutoFilterColorImages=false -dColorImageFilter=/FlateEncode \
-        -dPDFSETTINGS=/prepress -c save pop -f %s\""
+/* command line for calling GhostScript */
+#define CPGSCALL "%s -q -dCompatibilityLevel=%s -dNOPAUSE -dBATCH -dSAFER \
+	-sDEVICE=pdfwrite -sOutputFile=\"%s\" -dAutoRotatePages=/PageByPage \
+	-dAutoFilterColorImages=false -dColorImageFilter=/FlateEncode \
+	-dPDFSETTINGS=/prepress -c save pop -f %s"
 
 /* PDF version to be created - can be "1.5", "1.4", "1.3" or "1.2" */
 #define CPPDFVER "1.4"
@@ -124,3 +114,7 @@
    changing this can introduce security leaks if confidential
    information is processed! */
 #define CPUSERUMASK 0077
+
+/* group cups-pdf is supposed to run as - this will also be the gid for all
+   created directories and log files */
+#define CPGRP "lp"
